using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;

using AddimatDeviceManager;
using AddimatVirtualComPort;
using System.IO;


namespace Addimat_COM_Test_V04
{
	public partial class MainForm : Form
	{
		private VirtualComPort.TransmitMode gReadingMode;
		private VirtualComPort.TransmitMode gLastReadingMode;
		private bool gPolling;
		private VirtualComPort vcp;
    
    private String oldENQ1Num = "";
    private String oldENQ9Num = "";
		
		public MainForm()
		{
			InitializeComponent();
			Debug.Listeners.Clear();
			TextWriterTraceListener tr = new TextWriterTraceListener(System.IO.File.CreateText("Debug.txt"));
			Debug.Listeners.Add(tr);
			Debug.Listeners.Add(new TextWriterTraceListener(Console.Out));
			Debug.WriteLine("-------------------------------------");

			vcp = new VirtualComPort();
			vcp.PortReadEvent += new VirtualComPort.ReadEventHandler(OnPortData);
			vcp.PortDisconnectedEvent += new VirtualComPort.DisconnectedEventHandler(OnPortDisconnected);
			vcp.PortReconnectingEvent += new VirtualComPort.ReconnectingEventHandler(OnPortReconnecting);
			vcp.PortReconnectedEvent  += new VirtualComPort.ReconnectedEventHandler(OnPortReconnected);

			gPolling = false;
			rbStandard.Checked = true;
			gReadingMode = VirtualComPort.TransmitMode.Standard;
			setGUI_PollingMode();
			lbConnectState.Visible = false;
			
			timer1.Interval = 300;
			cbComPort.Click+= new System.EventHandler(this.cbComPort_Clicked);
			this.FormClosing+=new FormClosingEventHandler(MainForm_FormClosing);

			ScanPorts();
		}
		
	

		private void ScanPorts()
		{

			cbComPort.Items.Clear();
			Debug.WriteLine("Scan Ports: " + VirtualComPort.UsbDeviceName);
			String connectedComPorts = DeviceManager.GetConnectedComPorts(VirtualComPort.UsbDeviceName);
			Debug.WriteLine("Scan Ports: " + connectedComPorts);
			String[] ports = connectedComPorts.Split(new char[] { ';' });
			foreach (String port in ports)
			{
				if (port.Length > 0)
				{
					cbComPort.Items.Add(port);
				}
			}
		}

		private void OnPortDisconnected(String port) {
			
			timer1.Stop();
			Application.DoEvents();
			lbConnectState.Text = port + " disconnected.";
			lbConnectState.ForeColor=Color.Red;
			
			edt_Normal.Text = "";
			edt_Additional.Text = "";
			edtENQ9decimal.Text = "";
			
		}

		private void OnPortReconnecting(String port)
		{
			Application.DoEvents();
			lbConnectState.Text = port + " re-connecting ...";
			Application.DoEvents();
		}

		private void OnPortReconnected(String port) {
			Application.DoEvents();
			lbConnectState.Text = port +" connected.";
			lbConnectState.ForeColor = Color.Black;
			Application.DoEvents();
			if (gReadingMode == VirtualComPort.TransmitMode.Standard)
			{
				// Here, the last transaction has to be closed by doing the same as we 
				// would have received an "OUT" telegram.
			}
			timer1.Start();
		}

		private void OnPortData(VirtualComPort.TransmitMode mode, String text) {
			text = text.Replace('\r', ' ');
			text = text.Replace('\n', ' ');
			text = text.Trim();
      String dateTime = DateTime.Now.ToString(("yyyy.MM.dd HH:mm:ss")); 
			switch (mode)
			{
				case VirtualComPort.TransmitMode.Standard:		
					edt_Normal.Text = text;
          listBox1.Items.Add(dateTime + "   " + text);
          butSaveToFile.Enabled = true;
					break;
				case VirtualComPort.TransmitMode.Polling_ENQ1:
					edt_Normal.Text = text;
          if (oldENQ1Num.CompareTo(text) != 0)
          {
            listBox1.Items.Add(dateTime + "   " + text);
            butSaveToFile.Enabled = true;
            oldENQ1Num = text;
          }
					break;
				case VirtualComPort.TransmitMode.Polling_ENQ9:
					if (text.Length >= 20)
					{
						byte bh = byte.Parse(text.Substring(15, 2), System.Globalization.NumberStyles.HexNumber);
						byte bl = byte.Parse(text.Substring(18, 2), System.Globalization.NumberStyles.HexNumber);
						int value = bh * 256 + bl;
						edtENQ9decimal.Text = value.ToString();
					}
					edt_Additional.Text = text;
          if (oldENQ9Num.CompareTo(text) != 0)
          {
            listBox1.Items.Add(dateTime + "   " + text);
            butSaveToFile.Enabled = true;
            oldENQ9Num = text;
          }
					break;
			}
		}
		
	
		private void setGUI_PollingMode()
		{
			rbENQ1.Checked = true;

			lbNormal.Visible = true;
			edt_Normal.Visible = true;
			edt_Normal.Text = "";

			edt_Additional.Visible = false;
			edtENQ9decimal.Visible = false;
			lbAdditional.Visible = false;
			lbAdditional.Visible = false;

			gReadingMode = VirtualComPort.TransmitMode.Polling_ENQ1;

			
			if (gPolling){
				gReadingMode = VirtualComPort.TransmitMode.Polling_ENQ1;
				rbENQ1.Enabled = true;
				rbENQ9.Enabled = true;
				rbENQ1_ENQ9.Enabled = true;
				lbNormal.Text = "ENQ1";
				
			}
			else{
				gReadingMode = VirtualComPort.TransmitMode.Standard;
				rbENQ1.Enabled = false;			
				rbENQ9.Enabled = false;
				rbENQ1_ENQ9.Enabled = false;
				lbNormal.Text = "Standard";
			}
		}


		private void rbPolling_CheckedChanged(object sender, EventArgs e) {
			gPolling = rbPolling.Checked;
			setGUI_PollingMode();
		}

		private void rbStandard_CheckedChanged(object sender, EventArgs e)
		{
			gPolling = rbPolling.Checked;
			setGUI_PollingMode();
		}

		private void rbENQ1_CheckedChanged(object sender, EventArgs e)
		{
			if (rbENQ1.Checked)
			{
				gReadingMode = VirtualComPort.TransmitMode.Polling_ENQ1;
				
				edt_Normal.Visible = true;
				edt_Normal.Text = "";
				lbNormal.Visible = true;
				lbNormal.Text = "ENQ1";

				edt_Additional.Visible = false;
				edtENQ9decimal.Visible = false;
				lbAdditional.Visible = false;
			}
		}

		private void rbENQ9_CheckedChanged(object sender, EventArgs e)
		{
			if (rbENQ9.Checked)
			{
				gReadingMode = VirtualComPort.TransmitMode.Polling_ENQ9;
				
				edt_Additional.Visible = true;
				edt_Additional.Text = "";
				edtENQ9decimal.Visible = true;
				edtENQ9decimal.Text = "";
				edt_Additional.Text = "";
				lbAdditional.Visible = true;
				lbAdditional.Text = "ENQ9";

				edt_Normal.Visible = false;
				lbNormal.Visible = false;
			}
		}


		private void rbENQ1_ENQ9_CheckedChanged(object sender, EventArgs e)
		{
			if (rbENQ1_ENQ9.Checked)
			{
				gReadingMode = VirtualComPort.TransmitMode.Polling_Alternate;
				gLastReadingMode = VirtualComPort.TransmitMode.Polling_ENQ9;

				edt_Normal.Visible = true;
				edt_Normal.Text = "";
				lbNormal.Visible = true;
				lbNormal.Text = "ENQ1";

				edt_Additional.Visible = true;
				edt_Additional.Text = "";
				edtENQ9decimal.Visible = true;
				edtENQ9decimal.Text = "";
				lbAdditional.Visible = true;
				lbAdditional.Text = "ENQ9";
				
			}
		}

		private void timer1_Tick(object sender, EventArgs e)
		{

			try
			{
				if (gReadingMode == VirtualComPort.TransmitMode.Polling_Alternate)
				{
					if (gLastReadingMode == VirtualComPort.TransmitMode.Polling_ENQ1)
					{
						gLastReadingMode = VirtualComPort.TransmitMode.Polling_ENQ9;
					}
					else
					{
						gLastReadingMode = VirtualComPort.TransmitMode.Polling_ENQ1;
					}
					vcp.RequestData(gLastReadingMode);
				}
				else
				{
					vcp.RequestData(gReadingMode);
				}
			}
			catch (Exception ex)
			{
				Debug.WriteLine("MainForm: Timer1_tick Exception: "+ex.Message);
				return;
			}
		}

		private void cbComPort_Clicked(object sender, EventArgs e)
		{
			ScanPorts();
		}
		
		private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			try
			{
				Cursor.Current = Cursors.WaitCursor;
				timer1.Stop();
				
				Application.DoEvents();
				System.Threading.Thread.Sleep(500);
				vcp.CloseComPort();
				Application.DoEvents();
				System.Threading.Thread.Sleep(500);
				Cursor.Current = Cursors.Default;

				Debug.Flush();
				Debug.Close();
			}
			catch (Exception ex)
			{
				Debug.WriteLine("MainForm FormClosing Exception: " + ex.Message);
			}
		}  

		private void cbComPort_SelectedIndexChanged(object sender, EventArgs e)
		{
		
			timer1.Stop();
		    vcp.CloseComPort();
			
			string PortName=(string)cbComPort.SelectedItem;
            string errMsg = "";
			if (vcp.OpenComPort(PortName,ref errMsg) == false)
			{
				MessageBox.Show(PortName + " cannot be opened. Reason: "+errMsg,"Error",MessageBoxButtons.OK);
				return;
			}
			Application.DoEvents();
			lbConnectState.Text = PortName+ " connected.";
			lbConnectState.ForeColor = Color.Black;
			lbConnectState.Visible = true;
			Application.DoEvents();
			timer1.Start();
		}

    
    private void butSaveToFile_Click(object sender, EventArgs e)
    {
      timer1.Stop();
      String filename;
      SaveFileDialog saveFileDialog1 = new SaveFileDialog();

      saveFileDialog1.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*";
      saveFileDialog1.FilterIndex = 1;
      saveFileDialog1.RestoreDirectory = true;

      if (saveFileDialog1.ShowDialog() == DialogResult.OK)
      {
        try
        {
          filename = saveFileDialog1.FileName;
          TextWriter tw = new StreamWriter(filename);

          // write a line of text to the file
          foreach (String row in listBox1.Items)
          {
            tw.WriteLine(row);
          }
          // close the stream
          tw.Close();
          listBox1.Items.Clear();
          oldENQ1Num = "";
          oldENQ9Num = "";

        }
        catch (Exception ex)
        {
          MessageBox.Show("Write not possible. Reason: " + ex.Message, "Error", MessageBoxButtons.OK);
        }
      }
      timer1.Start();


    }
	}
}